import feedparser
import re


STATUS_OK = 0
STATUS_WARNING = 1
STATUS_CRITICAL = 2
STATUS_UNKNOWN = 3


def get_status(title, description):
    if title.startswith("Service is operating normally") \
       or title.startswith("Informational message: [RESOLVED]") \
       or title.startswith("Informational message: [Resolved]") \
       or re.search("he service is (now )?operating normally.?\n?", description) \
       or title.startswith("OK"):
        status = STATUS_OK
    elif title.startswith("Informational message") or title.startswith("Performanceissues"):
        status = STATUS_WARNING
    elif title.startswith("Service disruption"):
        status = STATUS_CRITICAL
    else:
        status = STATUS_UNKNOWN
    return status


class Feed(object):
    """
    This class wraps informations about a given AWS status feed, e.g. its
    URL, the monitored service, the region, etc.
    """
    def __init__(self, feed_url):
        self.url = feed_url
        self.__extract_elements()

    def __extract_elements(self):
        self.service = self.region = None
        feed_name = re.sub(r'.*/([^/]+?)\.rss', r'\1', self.url)
        #general case: a normal region at the end
        #ex: cloudfront-us-east-1
        #special case: pseudo-region "us-standard"
        #ex: s3-us-standard
        match = re.search(r'(.*)-(\w+-\w+-\d|us-standard)$', feed_name)
        if match:
            self.service = match.group(1)
            self.region = match.group(2)
        #special case: services with no region
        #ex: route53, or management-console
        else:
            self.service = feed_name

    def status(self):
        try:
            parsed = feedparser.parse(self.url)
            if parsed.entries:
                return parsed.entries[0]
            elif parsed['feed']['title']:
                return { "title": "OK but no event to display (empty feed)" }
        except KeyError:
            return { "title": "Unknown status, unable to parse feed ("+self.url+")" }



import re
from urllib2 import urlopen
from .feed import Feed

class StatusPage(object):
    """
    This class wraps access to Amazon status page and can be used to retrieve
    all available RSS feeds, and through these feeds monitored regions and
    services.
    It may be used to automatically discover new available feeds for instance,
    or for checking regions/services list.
    """
    BASE_URL = 'http://status.aws.amazon.com/'
    RSS_RE = r'href="/?(?P<feed_url>rss/[^"]+\.rss)"'

    def __init__(self):
        self.page_source = None
        self.feeds = set()
        self.rss_urls = set()
        self.services = set()
        self.regions = set()
        self.parse_page()

    def page(self):
        if not self.page_source:
            self.page_source = urlopen(self.BASE_URL)
        return self.page_source

    def parse_page(self):
        page_content = self.page().read()
        for match in re.finditer(self.RSS_RE, page_content):
            absolute_url = self.BASE_URL + match.group('feed_url')
            feed = Feed(absolute_url)
            self.feeds.add(feed)
            self.rss_urls.add(feed.url)
            if feed.service:
                self.services.add(feed.service)
            if feed.region:
                self.regions.add(feed.region)




import os

from setuptools import setup, find_packages

version = __import__('aws_status').__version__

def read(path):
    """Return the content of a file."""
    with open(path, 'r') as f:
        return f.read()

setup(
    name='aws-status',
    version=version,
    description='Wraps AWS status informations obtained via the status page.',
    long_description=(read('README.md')),
    url='http://github.com/jbbarth/aws-status',
    license='MIT',
    author='Jean-Baptiste Barth',
    author_email='jeanbaptiste.barth@gmail.com',
    packages=find_packages(exclude=['tests*']),
    scripts=[
        'bin/aws-status-check',
        'bin/aws-status-list',
    ],
    install_requires=[
        #TODO: read requirements.txt and deduce install_requires
        'feedparser>=5.1.3',
    ],
    setup_requires=[
        'feedparser>=5.1.3',
    ],
    test_suite="nose.collector",
    tests_require=[
        'nose',
    ],
    include_package_data=True,
    #see http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: System :: Monitoring',
    ],
)
